-------------------------------------------------------------------------------
-- Title      : VME64xCore test design for SVEC
-- Project    : VME64xCore
-- URL        : https://www.ohwr.org/projects/vme64x-core
-------------------------------------------------------------------------------
-- File       : svec_vmecore_test.vhd
-- Author(s)  : Tristan Gingold  <tristan.gingold@cern.ch>
-- Company    : CERN (BE-CO-HT)
-- Created    : 2017-09-19
-- Last update: 2017-10-09
-- Standard   : VHDL'93
-------------------------------------------------------------------------------
-- Description: Top-level file for the test design .
--
-------------------------------------------------------------------------------
-- Copyright (c) 2017 CERN
-------------------------------------------------------------------------------
-- GNU LESSER GENERAL PUBLIC LICENSE
--
-- This source file is free software; you can redistribute it
-- and/or modify it under the terms of the GNU Lesser General
-- Public License as published by the Free Software Foundation;
-- either version 2.1 of the License, or (at your option) any
-- later version.
--
-- This source is distributed in the hope that it will be
-- useful, but WITHOUT ANY WARRANTY; without even the implied
-- warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-- PURPOSE.  See the GNU Lesser General Public License for more
-- details.
--
-- You should have received a copy of the GNU Lesser General
-- Public License along with this source; if not, download it
-- from http://www.gnu.org/licenses/lgpl-2.1.html
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library work;
use work.gencores_pkg.all;
use work.wishbone_pkg.all;
use work.vme64x_pack.all;

library unisim;
use unisim.vcomponents.all;

entity svec_vmecore_test_top is
  port (
    ---------------------------------------------------------------------------
    -- Clocks/resets
    ---------------------------------------------------------------------------

    -- Reset from system fpga
    rst_n_i : in std_logic;

    -- Local oscillators
    clk_20m_vcxo_i : in std_logic;                -- 20MHz VCXO clock

--    clk_125m_pllref_p_i : in std_logic;           -- 125 MHz PLL reference
--    clk_125m_pllref_n_i : in std_logic;

--    clk_125m_gtp_n_i : in std_logic;              -- 125 MHz GTP reference
--    clk_125m_gtp_p_i : in std_logic;

    ---------------------------------------------------------------------------
    -- VME interface
    ---------------------------------------------------------------------------

    vme_write_n_i    : in    std_logic;
    vme_sysreset_n_i : in    std_logic;
    vme_retry_oe_o   : out   std_logic;
    vme_retry_n_o    : out   std_logic;
    vme_lword_n_b    : inout std_logic;
    vme_iackout_n_o  : out   std_logic;
    vme_iackin_n_i   : in    std_logic;
    vme_iack_n_i     : in    std_logic;
    vme_gap_i        : in    std_logic;
    vme_dtack_oe_o   : out   std_logic;
    vme_dtack_n_o    : out   std_logic;
    vme_ds_n_i       : in    std_logic_vector(1 downto 0);
    vme_data_oe_n_o  : out   std_logic;
    vme_data_dir_o   : out   std_logic;
    vme_berr_o       : out   std_logic;
    vme_as_n_i       : in    std_logic;
    vme_addr_oe_n_o  : out   std_logic;
    vme_addr_dir_o   : out   std_logic;
    vme_irq_n_o      : out   std_logic_vector(7 downto 1);
    vme_ga_i         : in    std_logic_vector(4 downto 0);
    vme_data_b       : inout std_logic_vector(31 downto 0);
    vme_am_i         : in    std_logic_vector(5 downto 0);
    vme_addr_b       : inout std_logic_vector(31 downto 1);

    ---------------------------------------------------------------------------
    -- SPI interfaces to DACs
    ---------------------------------------------------------------------------

--    pll20dac_din_o    : out std_logic;
--    pll20dac_sclk_o   : out std_logic;
--    pll20dac_sync_n_o : out std_logic;
--    pll25dac_din_o    : out std_logic;
--    pll25dac_sclk_o   : out std_logic;
--    pll25dac_sync_n_o : out std_logic;

    ---------------------------------------------------------------------------
    -- SFP I/O for transceiver
    ---------------------------------------------------------------------------

--    sfp_txp_o         : out   std_logic;
--    sfp_txn_o         : out   std_logic;
--    sfp_rxp_i         : in    std_logic;
--    sfp_rxn_i         : in    std_logic;
--    sfp_mod_def0_i    : in    std_logic;          -- sfp detect
--    sfp_mod_def1_b    : inout std_logic;          -- scl
--    sfp_mod_def2_b    : inout std_logic;          -- sda
--    sfp_rate_select_o : out   std_logic;
--    sfp_tx_fault_i    : in    std_logic;
--    sfp_tx_disable_o  : out   std_logic;
--    sfp_los_i         : in    std_logic;

    ---------------------------------------------------------------------------
    -- Carrier I2C EEPROM
    ---------------------------------------------------------------------------

--    carrier_scl_b : inout std_logic;
--    carrier_sda_b : inout std_logic;

    ---------------------------------------------------------------------------
    -- Onewire interface
    ---------------------------------------------------------------------------

--    onewire_b : inout std_logic;

    ---------------------------------------------------------------------------
    -- UART
    ---------------------------------------------------------------------------

--    uart_rxd_i : in  std_logic;
--    uart_txd_o : out std_logic;

    ---------------------------------------------------------------------------
    -- SPI (flash is connected to SFPGA and routed to AFPGA
    -- once the boot process is complete)
    ---------------------------------------------------------------------------

--    spi_sclk_o : out std_logic;
--    spi_ncs_o  : out std_logic;
--    spi_mosi_o : out std_logic;
--    spi_miso_i : in  std_logic;

    ---------------------------------------------------------------------------
    -- Carrier front panel LEDs and IOs
    ---------------------------------------------------------------------------

    fp_led_line_oen_o : out std_logic_vector(1 downto 0);
    fp_led_line_o     : out std_logic_vector(1 downto 0);
    fp_led_column_o   : out std_logic_vector(3 downto 0)

--    fp_gpio1_o      : out std_logic;              -- PPS output
--    fp_gpio2_o      : out std_logic;              -- Ref clock div2 output
--    fp_gpio3_i      : in  std_logic;              -- ext 10MHz clock input
--    fp_gpio4_i      : in  std_logic;              -- ext PPS intput
--    fp_term_en_o    : out std_logic_vector(4 downto 1);
--    fp_gpio1_a2b_o  : out std_logic;
--    fp_gpio2_a2b_o  : out std_logic;
--    fp_gpio34_a2b_o : out std_logic
    );

end entity svec_vmecore_test_top;

architecture top of svec_vmecore_test_top is

  -- Wishbone bus from master
  signal master_out : t_wishbone_master_out;
  signal master_in  : t_wishbone_master_in;

  -- clock and reset
  signal clk_sys_62m5   : std_logic;
  signal rst_sys_62m5_n : std_logic;
  signal clk_ref_125m   : std_logic;
  signal clk_ref_div2   : std_logic;
  signal clk_ext_ref    : std_logic;

  -- VME
  signal vme_data_b_out    : std_logic_vector(31 downto 0);
  signal vme_addr_b_out    : std_logic_vector(31 downto 1);
  signal vme_lword_n_b_out : std_logic;
  signal Vme_data_dir_int  : std_logic;
  signal vme_addr_dir_int  : std_logic;
  signal vme_ga            : std_logic_vector(5 downto 0);

  -- LEDs and GPIO
  signal pps         : std_logic;
  signal pps_led     : std_logic;
  signal pps_ext_in  : std_logic;
  signal svec_led    : std_logic_vector(15 downto 0);

  signal pllout_clk_fb_sys, pllout_clk_sys : std_logic;
  signal clk_20m_vcxo_buf                  : std_logic;
  signal clk_sys                           : std_logic;
  signal local_reset_n                     : std_logic;

  signal powerup_reset_cnt : unsigned(7 downto 0) := "00000000";
  signal powerup_rst_n     : std_logic            := '0';
  signal sys_locked        : std_logic;

  signal s_irq : std_logic;
begin  -- architecture top

  p_powerup_reset : process(clk_sys)
  begin
    if rising_edge(clk_sys) then
      if(vme_sysreset_n_i = '0' or rst_n_i = '0') then
        powerup_rst_n <= '0';
      elsif sys_locked = '1' then
        if(powerup_reset_cnt = "11111111") then
          powerup_rst_n <= '1';
        else
          powerup_rst_n     <= '0';
          powerup_reset_cnt <= powerup_reset_cnt + 1;
        end if;
      else
        powerup_rst_n     <= '0';
        powerup_reset_cnt <= "00000000";
      end if;
    end if;
  end process;


-------------------------------------------------------------------------------
-- Clock distribution/PLL and reset
-------------------------------------------------------------------------------

  --  Input is 20Mhz
  U_cmp_sys_pll : PLL_BASE
    generic map (
      BANDWIDTH          => "OPTIMIZED",
      CLK_FEEDBACK       => "CLKFBOUT",
      COMPENSATION       => "INTERNAL",
      DIVCLK_DIVIDE      => 1,
      CLKFBOUT_MULT      => 50,  -- 1Ghz
      CLKFBOUT_PHASE     => 0.000,
      CLKOUT0_DIVIDE     => 8,         -- 62.5 MHz
      CLKOUT0_PHASE      => 0.000,
      CLKOUT0_DUTY_CYCLE => 0.500,
      CLKOUT1_DIVIDE     => 8,         -- 62.5 MHz
      CLKOUT1_PHASE      => 0.000,
      CLKOUT1_DUTY_CYCLE => 0.500,
      CLKOUT2_DIVIDE     => 8,
      CLKOUT2_PHASE      => 0.000,
      CLKOUT2_DUTY_CYCLE => 0.500,
      CLKIN_PERIOD       => 50.0,
      REF_JITTER         => 0.016)
    port map (
      CLKFBOUT => pllout_clk_fb_sys,
      CLKOUT0  => pllout_clk_sys,
      CLKOUT1  => open,                 -- pllout_clk_sys,
      CLKOUT2  => open,
      CLKOUT3  => open,
      CLKOUT4  => open,
      CLKOUT5  => open,
      LOCKED   => sys_locked,
      RST      => '0',
      CLKFBIN  => pllout_clk_fb_sys,
      CLKIN    => clk_20m_vcxo_buf);


  U_Sync_Reset : gc_sync_ffs
    port map (
      clk_i    => clk_sys,
      rst_n_i  => '1',
      data_i   => powerup_rst_n,
      synced_o => local_reset_n);

  U_cmp_clk_vcxo_buf : BUFG
    port map (
      O => clk_20m_vcxo_buf,
      I => clk_20m_vcxo_i);

  U_cmp_clk_sys_buf : BUFG
    port map (
      O => clk_sys,
      I => pllout_clk_sys);

  -----------------------------------------------------------------------------
  -- VME64x Core and buffers
  -----------------------------------------------------------------------------

  inst_vme_core : vme64xcore_top
    generic map (
      g_CLOCK_PERIOD => 8,
      g_USER_CSR_EXT => False)
    port map (
      clk_i           => clk_sys,
      rst_n_i         => local_reset_n,
      VME_AS_n_i      => vme_as_n_i,
      VME_RST_n_i     => vme_sysreset_n_i,
      VME_WRITE_n_i   => vme_write_n_i,
      VME_AM_i        => vme_am_i,
      VME_DS_n_i      => vme_ds_n_i,
      VME_GA_i        => vme_ga,
      VME_BERR_o      => vme_berr_o,
      VME_DTACK_n_o   => vme_dtack_n_o,
      VME_RETRY_n_o   => vme_retry_n_o,
      VME_RETRY_OE_o  => vme_retry_oe_o,
      VME_LWORD_n_i   => vme_lword_n_b,
      VME_LWORD_n_o   => vme_lword_n_b_out,
      VME_ADDR_i      => vme_addr_b,
      VME_DATA_o      => vme_data_b_out,
      VME_ADDR_o      => vme_addr_b_out,
      VME_DATA_i      => vme_data_b,
      VME_IRQ_o       => vme_irq_n_o,
      VME_IACK_n_i    => vme_iack_n_i,
      VME_IACKIN_n_i  => vme_iackin_n_i,
      VME_IACKOUT_n_o => vme_iackout_n_o,
      VME_DTACK_OE_o  => vme_dtack_oe_o,
      VME_DATA_DIR_o  => vme_data_dir_int,
      VME_DATA_OE_N_o => vme_data_oe_n_o,
      VME_ADDR_DIR_o  => vme_addr_dir_int,
      VME_ADDR_OE_N_o => vme_addr_oe_n_o,
      DAT_i   => master_in.dat,
      DAT_o   => master_out.dat,
      ADR_o   => master_out.adr,
      CYC_o   => master_out.cyc,
      ERR_i   => master_in.err,
      SEL_o   => master_out.sel,
      STB_o   => master_out.stb,
      ACK_i   => master_in.ack,
      WE_o    => master_out.we,
      STALL_i => master_in.stall,

      irq_i           =>  s_irq);

  vme_ga <= vme_gap_i & vme_ga_i;

  -- VME tri-state buffers
  vme_data_b    <= vme_data_b_out    when vme_data_dir_int = '1'
                   else (others => 'Z');
  vme_addr_b    <= vme_addr_b_out    when vme_addr_dir_int = '1'
                   else (others => 'Z');
  vme_lword_n_b <= vme_lword_n_b_out when vme_addr_dir_int = '1'
                   else 'Z';

  vme_addr_dir_o <= vme_addr_dir_int;
  vme_data_dir_o <= vme_data_dir_int;

  -- tri-state Carrier EEPROM
--  carrier_sda_b <= 'Z';
--  carrier_scl_b <= 'Z';

  -- Tristates for SFP EEPROM
--  sfp_mod_def1_b <= 'Z';
--  sfp_mod_def2_b <= 'Z';

  -- tri-state onewire access
--  onewire_b    <= 'Z';

  ------------------------------------------------------------------------------
  -- Carrier front panel LEDs and LEMOs
  ------------------------------------------------------------------------------

  cmp_led_controller : gc_bicolor_led_ctrl
    generic map(
      g_nb_column    => 4,
      g_nb_line      => 2,
      g_clk_freq     => 62_500_000,               -- in Hz
      g_refresh_rate => 250                       -- in Hz
      )
    port map(
      rst_n_i => local_reset_n,
      clk_i   => clk_sys,

      led_intensity_i => "1100100",               -- in %

      led_state_i => svec_led,

      column_o   => fp_led_column_o,
      line_o     => fp_led_line_o,
      line_oen_o => fp_led_line_oen_o);

  inst_test: entity work.vmecore_test
    port map (clk_sys_i => clk_sys,
              rst_n_i => local_reset_n,
              slave_i => master_out,
              slave_o => master_in,
              leds_o => svec_led,
              irq_o => s_irq);
end architecture top;
